/* automatically generated from xfdesktop-settings-appearance-frame-ui.glade */
#ifdef __SUNPRO_C
#pragma align 4 (xfdesktop_settings_appearance_frame_ui)
#endif
#ifdef __GNUC__
static const char xfdesktop_settings_appearance_frame_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfdesktop_settings_appearance_frame_ui[] =
#endif
{
  "<?xml version=\"1.0\"?>\n<!--Generated with glade3 3.4.5 on Sun Nov 16 "
  "16:39:43 2008 -->\n<interface>\n  <object class=\"GtkAdjustment\" id=\""
  "adjustment3\">\n    <property name=\"upper\">65535</property>\n    <pro"
  "perty name=\"lower\">1</property>\n    <property name=\"page_increment\""
  ">10</property>\n    <property name=\"step_increment\">1</property>\n   "
  " <property name=\"page_size\">0</property>\n    <property name=\"value\""
  ">10</property>\n  </object>\n  <object class=\"GtkListStore\" id=\"mode"
  "l2\">\n    <columns>\n      <column type=\"gchararray\"/>\n    </column"
  "s>\n    <data>\n      <row>\n        <col id=\"0\" translatable=\"yes\""
  ">Solid color</col>\n      </row>\n      <row>\n        <col id=\"0\" tr"
  "anslatable=\"yes\">Horizontal gradient</col>\n      </row>\n      <row>"
  "\n        <col id=\"0\" translatable=\"yes\">Vertical gradient</col>\n "
  "     </row>\n      <row>\n        <col id=\"0\" translatable=\"yes\">Tr"
  "ansparent</col>\n      </row>\n    </data>\n  </object>\n  <object clas"
  "s=\"GtkAlignment\" id=\"alignment_settings\">\n    <property name=\"vis"
  "ible\">True</property>\n    <property name=\"events\">GDK_POINTER_MOTIO"
  "N_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUT"
  "TON_RELEASE_MASK</property>\n    <property name=\"border_width\">12</pr"
  "operty>\n    <property name=\"left_padding\">6</property>\n    <child>\n"
  "      <object class=\"GtkVBox\" id=\"background_tab_vbox\">\n        <p"
  "roperty name=\"visible\">True</property>\n        <property name=\"spac"
  "ing\">6</property>\n        <child>\n          <object class=\"GtkInfoB"
  "ar\" id=\"infobar_header\">\n            <property name=\"visible\">Tru"
  "e</property>\n            <property name=\"events\">GDK_POINTER_MOTION_"
  "MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTO"
  "N_RELEASE_MASK</property>\n            <property name=\"message-type\">"
  "GTK_MESSAGE_INFO</property>\n          </object>\n          <packing>\n"
  "            <property name=\"expand\">False</property>\n            <pr"
  "operty name=\"position\">0</property>\n          </packing>\n        </"
  "child>\n        <child>\n          <object class=\"GtkHBox\" id=\"backg"
  "round_tab_hbox\">\n            <property name=\"visible\">True</propert"
  "y>\n            <child>\n              <object class=\"GtkLabel\" id=\""
  "label_header\">\n                <property name=\"visible\">True</prope"
  "rty>\n                <property name=\"events\">GDK_POINTER_MOTION_MASK"
  " | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RE"
  "LEASE_MASK</property>\n              </object>\n              <packing>"
  "\n                <property name=\"expand\">False</property>\n         "
  "       <property name=\"position\">1</property>\n              </packin"
  "g>\n            </child>\n          </object>\n          <packing>\n   "
  "         <property name=\"expand\">False</property>\n            <prope"
  "rty name=\"position\">1</property>\n          </packing>\n        </chi"
  "ld>\n        <child>\n          <object class=\"GtkScrolledWindow\" id="
  "\"iconview_imagelist_scrolled_window\">\n            <property name=\"v"
  "isible\">True</property>\n            <property name=\"can_focus\">True"
  "</property>\n            <property name=\"events\">GDK_POINTER_MOTION_M"
  "ASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON"
  "_RELEASE_MASK</property>\n            <property name=\"hscrollbar_polic"
  "y\">GTK_POLICY_AUTOMATIC</property>\n            <property name=\"vscro"
  "llbar_policy\">GTK_POLICY_AUTOMATIC</property>\n            <property n"
  "ame=\"shadow_type\">GTK_SHADOW_ETCHED_IN</property>\n            <child"
  ">\n              <object class=\"GtkIconView\" id=\"iconview_imagelist\""
  ">\n                <property name=\"visible\">True</property>\n        "
  "        <property name=\"can_focus\">True</property>\n                <"
  "property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_H"
  "INT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n"
  "              </object>\n            </child>\n          </object>\n   "
  "       <packing>\n            <property name=\"expand\">True</property>"
  "\n            <property name=\"position\">2</property>\n          </pac"
  "king>\n        </child>\n        <child>\n          <object class=\"Gtk"
  "Table\" id=\"folder_color_and_image_style_table\">\n            <proper"
  "ty name=\"visible\">True</property>\n            <property name=\"n-col"
  "umns\">8</property>\n            <property name=\"n-rows\">2</property>"
  "\n            <property name=\"row-spacing\">6</property>\n            "
  "<property name=\"column-spacing\">12</property>\n            <child>\n "
  "             <object class=\"GtkLabel\" id=\"label_folder\">\n         "
  "       <property name=\"visible\">True</property>\n                <pro"
  "perty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT"
  "_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n   "
  "             <property name=\"label\" translatable=\"yes\">_Folder:</pr"
  "operty>\n                <property name=\"use_underline\">True</propert"
  "y>\n                <property name=\"mnemonic_widget\">btn_folder</prop"
  "erty>\n              </object>\n              <packing>\n              "
  "  <property name=\"left-attach\">0</property>\n                <propert"
  "y name=\"right-attach\">1</property>\n                <property name=\""
  "top-attach\">0</property>\n                <property name=\"bottom-atta"
  "ch\">1</property>\n                <property name=\"x-options\">GTK_SHR"
  "INK</property>\n                <property name=\"y-options\">GTK_SHRINK"
  "</property>\n              </packing>\n            </child>\n          "
  "  <child>\n              <object class=\"GtkFileChooserButton\" id=\"bt"
  "n_folder\">\n                <property name=\"visible\">True</property>"
  "\n                <property name=\"can_focus\">True</property>\n       "
  "         <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTE"
  "R_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</p"
  "roperty>\n                <property name=\"tooltip-text\" translatable="
  "\"yes\">Choose the folder to select wallpapers from.</property>\n      "
  "          <property name=\"action\">GTK_FILE_CHOOSER_ACTION_SELECT_FOLD"
  "ER</property>\n              </object>\n              <packing>\n      "
  "          <property name=\"left-attach\">1</property>\n                "
  "<property name=\"right-attach\">2</property>\n                <property"
  " name=\"top-attach\">0</property>\n                <property name=\"bot"
  "tom-attach\">1</property>\n                <property name=\"x-options\""
  ">GTK_FILL|GTK_EXPAND</property>\n                <property name=\"y-opt"
  "ions\">GTK_SHRINK</property>\n              </packing>\n            </c"
  "hild>\n            <child>\n              <object class=\"GtkAlignment\""
  " id=\"alignment_spacer\">\n                <property name=\"visible\">T"
  "rue</property>\n              </object>\n              <packing>\n     "
  "           <property name=\"left-attach\">4</property>\n               "
  " <property name=\"right-attach\">5</property>\n                <propert"
  "y name=\"top-attach\">0</property>\n                <property name=\"bo"
  "ttom-attach\">1</property>\n                <property name=\"x-options\""
  ">GTK_FILL|GTK_EXPAND</property>\n                <property name=\"y-opt"
  "ions\">GTK_SHRINK</property>\n              </packing>\n            </c"
  "hild>\n            <child>\n              <object class=\"GtkLabel\" id"
  "=\"label_style\">\n                <property name=\"visible\">True</pro"
  "perty>\n                <property name=\"events\">GDK_POINTER_MOTION_MA"
  "SK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_"
  "RELEASE_MASK</property>\n                <property name=\"label\" trans"
  "latable=\"yes\">St_yle:</property>\n                <property name=\"us"
  "e_underline\">True</property>\n                <property name=\"mnemoni"
  "c_widget\">combo_style</property>\n              </object>\n           "
  "   <packing>\n                <property name=\"left-attach\">5</propert"
  "y>\n                <property name=\"right-attach\">6</property>\n     "
  "           <property name=\"top-attach\">0</property>\n                "
  "<property name=\"bottom-attach\">1</property>\n                <propert"
  "y name=\"x-options\">GTK_SHRINK</property>\n                <property n"
  "ame=\"y-options\">GTK_SHRINK</property>\n              </packing>\n    "
  "        </child>\n            <child>\n              <object class=\"Gt"
  "kComboBoxText\" id=\"combo_style\">\n                <property name=\"v"
  "isible\">True</property>\n                <property name=\"events\">GDK"
  "_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_"
  "MASK | GDK_BUTTON_RELEASE_MASK</property>\n                <property na"
  "me=\"tooltip-text\" translatable=\"yes\">Specify how the image will be "
  "resized to fit the screen.</property>\n                <items>\n       "
  "           <item translatable=\"yes\">None</item>\n                  <i"
  "tem translatable=\"yes\">Centered</item>\n                  <item trans"
  "latable=\"yes\">Tiled</item>\n                  <item translatable=\"ye"
  "s\">Stretched</item>\n                  <item translatable=\"yes\">Scal"
  "ed</item>\n                  <item translatable=\"yes\">Zoomed</item>\n"
  "                  <item translatable=\"yes\">Spanning Screens</item>\n "
  "               </items>\n              </object>\n              <packin"
  "g>\n                <property name=\"left-attach\">6</property>\n      "
  "          <property name=\"right-attach\">7</property>\n               "
  " <property name=\"top-attach\">0</property>\n                <property "
  "name=\"bottom-attach\">1</property>\n                <property name=\"x"
  "-options\">GTK_FILL</property>\n                <property name=\"y-opti"
  "ons\">GTK_SHRINK</property>\n              </packing>\n            </ch"
  "ild>\n            <child>\n              <object class=\"GtkLabel\" id="
  "\"label_color\">\n                <property name=\"visible\">True</prop"
  "erty>\n                <property name=\"events\">GDK_POINTER_MOTION_MAS"
  "K | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_R"
  "ELEASE_MASK</property>\n                <property name=\"label\" transl"
  "atable=\"yes\">C_olor:</property>\n                <property name=\"use"
  "_underline\">True</property>\n                <property name=\"mnemonic"
  "_widget\">combo_colors</property>\n              </object>\n           "
  "   <packing>\n                <property name=\"left-attach\">0</propert"
  "y>\n                <property name=\"right-attach\">1</property>\n     "
  "           <property name=\"top-attach\">1</property>\n                "
  "<property name=\"bottom-attach\">2</property>\n                <propert"
  "y name=\"x-options\">GTK_SHRINK</property>\n                <property n"
  "ame=\"y-options\">GTK_SHRINK</property>\n              </packing>\n    "
  "        </child>\n            <child>\n              <object class=\"Gt"
  "kComboBox\" id=\"combo_colors\">\n                <property name=\"visi"
  "ble\">True</property>\n                <property name=\"events\">GDK_PO"
  "INTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MAS"
  "K | GDK_BUTTON_RELEASE_MASK</property>\n                <property name="
  "\"tooltip-text\" translatable=\"yes\">Specify the style of the color dr"
  "awn behind the backdrop image.</property>\n                <property na"
  "me=\"model\">model2</property>\n                <child>\n              "
  "    <object class=\"GtkCellRendererText\" id=\"renderer2\"/>\n         "
  "         <attributes>\n                    <attribute name=\"text\">0</"
  "attribute>\n                  </attributes>\n                </child>\n"
  "              </object>\n              <packing>\n                <prop"
  "erty name=\"left-attach\">1</property>\n                <property name="
  "\"right-attach\">2</property>\n                <property name=\"top-att"
  "ach\">1</property>\n                <property name=\"bottom-attach\">2<"
  "/property>\n                <property name=\"x-options\">GTK_FILL|GTK_E"
  "XPAND</property>\n                <property name=\"y-options\">GTK_SHRI"
  "NK</property>\n              </packing>\n            </child>\n        "
  "    <child>\n              <object class=\"GtkColorButton\" id=\"color1"
  "_btn\">\n                <property name=\"visible\">True</property>\n  "
  "              <property name=\"can_focus\">True</property>\n           "
  "     <property name=\"receives_default\">True</property>\n             "
  "   <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTI"
  "ON_HINT_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUT"
  "TON_RELEASE_MASK</property>\n                <property name=\"tooltip-t"
  "ext\" translatable=\"yes\">Specifies the solid color, or the \"left\" o"
  "r \"top\" color of the gradient.</property>\n                <property "
  "name=\"title\" translatable=\"yes\">Select First Color</property>\n    "
  "            <property name=\"color\">#151522223333</property>\n        "
  "      </object>\n              <packing>\n                <property nam"
  "e=\"left-attach\">2</property>\n                <property name=\"right-"
  "attach\">3</property>\n                <property name=\"top-attach\">1<"
  "/property>\n                <property name=\"bottom-attach\">2</propert"
  "y>\n                <property name=\"x-options\">GTK_SHRINK</property>\n"
  "                <property name=\"y-options\">GTK_SHRINK</property>\n   "
  "           </packing>\n            </child>\n            <child>\n     "
  "         <object class=\"GtkColorButton\" id=\"color2_btn\">\n         "
  "       <property name=\"visible\">True</property>\n                <pro"
  "perty name=\"can_focus\">True</property>\n                <property nam"
  "e=\"receives_default\">True</property>\n                <property name="
  "\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK"
  "_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</"
  "property>\n                <property name=\"tooltip-text\" translatable"
  "=\"yes\">Specifies the \"right\" or \"bottom\" color of the gradient.</"
  "property>\n                <property name=\"title\" translatable=\"yes\""
  ">Select Second Color</property>\n                <property name=\"color"
  "\">#151522223333</property>\n              </object>\n              <pa"
  "cking>\n                <property name=\"left-attach\">3</property>\n  "
  "              <property name=\"right-attach\">4</property>\n           "
  "     <property name=\"top-attach\">1</property>\n                <prope"
  "rty name=\"bottom-attach\">2</property>\n                <property name"
  "=\"x-options\">GTK_SHRINK</property>\n                <property name=\""
  "y-options\">GTK_SHRINK</property>\n              </packing>\n          "
  "  </child>\n            <child>\n              <object class=\"GtkCheck"
  "Button\" id=\"chk_apply_to_all\">\n                <property name=\"vis"
  "ible\">True</property>\n                <property name=\"sensitive\">Tr"
  "ue</property>\n                <property name=\"can_focus\">True</prope"
  "rty>\n                <property name=\"events\">GDK_POINTER_MOTION_MASK"
  " | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RE"
  "LEASE_MASK</property>\n                <property name=\"label\" transla"
  "table=\"yes\">Apply to all _workspaces</property>\n                <pro"
  "perty name=\"use_underline\">True</property>\n                <property"
  " name=\"draw_indicator\">True</property>\n              </object>\n    "
  "          <packing>\n                <property name=\"left-attach\">6</"
  "property>\n                <property name=\"right-attach\">7</property>"
  "\n                <property name=\"top-attach\">1</property>\n         "
  "       <property name=\"bottom-attach\">2</property>\n                <"
  "property name=\"x-options\">GTK_SHRINK</property>\n                <pro"
  "perty name=\"y-options\">GTK_SHRINK</property>\n              </packing"
  ">\n            </child>\n          </object>\n          <packing>\n    "
  "        <property name=\"expand\">False</property>\n            <proper"
  "ty name=\"position\">3</property>\n          </packing>\n        </chil"
  "d>\n        <child>\n          <object class=\"GtkHBox\" id=\"cycle_opt"
  "ions_hbox\">\n            <property name=\"visible\">True</property>\n "
  "           <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POIN"
  "TER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK<"
  "/property>\n            <property name=\"spacing\">12</property>\n     "
  "       <child>\n              <object class=\"GtkCheckButton\" id=\"chk"
  "_cycle_backdrop\">\n                <property name=\"visible\">True</pr"
  "operty>\n                <property name=\"sensitive\">True</property>\n"
  "                <property name=\"can_focus\">True</property>\n         "
  "       <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_"
  "MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</pro"
  "perty>\n                <property name=\"label\" translatable=\"yes\">C"
  "hange the _background </property>\n                <property name=\"too"
  "ltip-text\" translatable=\"yes\">Automatically select a different backg"
  "round from the current directory.</property>\n                <property"
  " name=\"use_underline\">True</property>\n                <property name"
  "=\"draw_indicator\">True</property>\n              </object>\n         "
  "     <packing>\n                <property name=\"expand\">False</proper"
  "ty>\n              </packing>\n            </child>\n            <child"
  ">\n              <object class=\"GtkComboBoxText\" id=\"combo_cycle_bac"
  "kdrop_period\">\n                <property name=\"visible\">True</prope"
  "rty>\n                <property name=\"sensitive\">False</property>\n  "
  "              <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_P"
  "OINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MA"
  "SK</property>\n                <property name=\"tooltip-text\" translat"
  "able=\"yes\">Specify how often the background will change.</property>\n"
  "                <items>\n                  <item translatable=\"yes\">i"
  "n seconds:</item>\n                  <item translatable=\"yes\">in minu"
  "tes:</item>\n                  <item translatable=\"yes\">in hours:</it"
  "em>\n                  <item translatable=\"yes\">at start up</item>\n "
  "                 <item translatable=\"yes\">every hour</item>\n        "
  "          <item translatable=\"yes\">every day</item>\n                "
  "  <item translatable=\"yes\">chronologically</item>\n                </"
  "items>\n              </object>\n              <packing>\n             "
  "   <property name=\"expand\">False</property>\n              </packing>"
  "\n            </child>\n            <child>\n              <object clas"
  "s=\"GtkSpinButton\" id=\"spin_backdrop_time\">\n                <proper"
  "ty name=\"visible\">True</property>\n                <property name=\"s"
  "ensitive\">False</property>\n                <property name=\"can_focus"
  "\">True</property>\n                <property name=\"events\">GDK_POINT"
  "ER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK |"
  " GDK_BUTTON_RELEASE_MASK</property>\n                <property name=\"a"
  "djustment\">adjustment3</property>\n                <property name=\"to"
  "oltip-text\" translatable=\"yes\">Amount of time before a different bac"
  "kground is selected.</property>\n              </object>\n             "
  " <packing>\n                <property name=\"expand\">False</property>\n"
  "                <property name=\"position\">2</property>\n             "
  " </packing>\n            </child>\n            <child>\n              <"
  "object class=\"GtkCheckButton\" id=\"chk_random_backdrop_order\">\n    "
  "            <property name=\"visible\">True</property>\n               "
  " <property name=\"sensitive\">False</property>\n                <proper"
  "ty name=\"can_focus\">True</property>\n                <property name=\""
  "events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_B"
  "UTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                "
  "<property name=\"label\" translatable=\"yes\">_Random Order</property>\n"
  "                <property name=\"use_underline\">True</property>\n     "
  "           <property name=\"draw_indicator\">True</property>\n         "
  "       <property name=\"tooltip-text\" translatable=\"yes\">Randomly se"
  "lects another image from the same directory when the wallpaper is to cy"
  "cle.</property>\n              </object>\n              <packing>\n    "
  "            <property name=\"expand\">False</property>\n              <"
  "/packing>\n            </child>\n          </object>\n          <packin"
  "g>\n            <property name=\"expand\">False</property>\n           "
  " <property name=\"position\">5</property>\n          </packing>\n      "
  "  </child>\n      </object>\n    </child>\n  </object>\n</interface>\n"
};

static const unsigned xfdesktop_settings_appearance_frame_ui_length = 19451u;

