/* Generated by dbus-binding-tool; do not edit! */

#include <glib.h>
#include <dbus/dbus-glib.h>

G_BEGIN_DECLS

#ifndef _DBUS_GLIB_ASYNC_DATA_FREE
#define _DBUS_GLIB_ASYNC_DATA_FREE
static
#ifdef G_HAVE_INLINE
inline
#endif
void
_dbus_glib_async_data_free (gpointer stuff)
{
	g_slice_free (DBusGAsyncData, stuff);
}
#endif

#ifndef DBUS_GLIB_CLIENT_WRAPPERS_xfdesktop_trash_proxy
#define DBUS_GLIB_CLIENT_WRAPPERS_xfdesktop_trash_proxy

static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
xfdesktop_trash_proxy_display_trash (DBusGProxy *proxy, const char * IN_display, const char * IN_startup_id, GError **error)

{
  return dbus_g_proxy_call (proxy, "DisplayTrash", error, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID, G_TYPE_INVALID);
}

typedef void (*xfdesktop_trash_proxy_display_trash_reply) (DBusGProxy *proxy, GError *error, gpointer userdata);

static void
xfdesktop_trash_proxy_display_trash_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
  (*(xfdesktop_trash_proxy_display_trash_reply)data->cb) (proxy, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
xfdesktop_trash_proxy_display_trash_async (DBusGProxy *proxy, const char * IN_display, const char * IN_startup_id, xfdesktop_trash_proxy_display_trash_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "DisplayTrash", xfdesktop_trash_proxy_display_trash_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID);
}
static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
xfdesktop_trash_proxy_empty_trash (DBusGProxy *proxy, const char * IN_display, const char * IN_startup_id, GError **error)

{
  return dbus_g_proxy_call (proxy, "EmptyTrash", error, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID, G_TYPE_INVALID);
}

typedef void (*xfdesktop_trash_proxy_empty_trash_reply) (DBusGProxy *proxy, GError *error, gpointer userdata);

static void
xfdesktop_trash_proxy_empty_trash_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
  (*(xfdesktop_trash_proxy_empty_trash_reply)data->cb) (proxy, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
xfdesktop_trash_proxy_empty_trash_async (DBusGProxy *proxy, const char * IN_display, const char * IN_startup_id, xfdesktop_trash_proxy_empty_trash_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "EmptyTrash", xfdesktop_trash_proxy_empty_trash_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID);
}
static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
xfdesktop_trash_proxy_move_to_trash (DBusGProxy *proxy, const char ** IN_filenames, const char * IN_display, const char * IN_startup_id, GError **error)

{
  return dbus_g_proxy_call (proxy, "MoveToTrash", error, G_TYPE_STRV, IN_filenames, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID, G_TYPE_INVALID);
}

typedef void (*xfdesktop_trash_proxy_move_to_trash_reply) (DBusGProxy *proxy, GError *error, gpointer userdata);

static void
xfdesktop_trash_proxy_move_to_trash_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
  (*(xfdesktop_trash_proxy_move_to_trash_reply)data->cb) (proxy, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
xfdesktop_trash_proxy_move_to_trash_async (DBusGProxy *proxy, const char ** IN_filenames, const char * IN_display, const char * IN_startup_id, xfdesktop_trash_proxy_move_to_trash_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "MoveToTrash", xfdesktop_trash_proxy_move_to_trash_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_STRV, IN_filenames, G_TYPE_STRING, IN_display, G_TYPE_STRING, IN_startup_id, G_TYPE_INVALID);
}
static
#ifdef G_HAVE_INLINE
inline
#endif
gboolean
xfdesktop_trash_proxy_query_trash (DBusGProxy *proxy, gboolean* OUT_full, GError **error)

{
  return dbus_g_proxy_call (proxy, "QueryTrash", error, G_TYPE_INVALID, G_TYPE_BOOLEAN, OUT_full, G_TYPE_INVALID);
}

typedef void (*xfdesktop_trash_proxy_query_trash_reply) (DBusGProxy *proxy, gboolean OUT_full, GError *error, gpointer userdata);

static void
xfdesktop_trash_proxy_query_trash_async_callback (DBusGProxy *proxy, DBusGProxyCall *call, void *user_data)
{
  DBusGAsyncData *data = (DBusGAsyncData*) user_data;
  GError *error = NULL;
  gboolean OUT_full;
  dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_BOOLEAN, &OUT_full, G_TYPE_INVALID);
  (*(xfdesktop_trash_proxy_query_trash_reply)data->cb) (proxy, OUT_full, error, data->userdata);
  return;
}

static
#ifdef G_HAVE_INLINE
inline
#endif
DBusGProxyCall*
xfdesktop_trash_proxy_query_trash_async (DBusGProxy *proxy, xfdesktop_trash_proxy_query_trash_reply callback, gpointer userdata)

{
  DBusGAsyncData *stuff;
  stuff = g_slice_new (DBusGAsyncData);
  stuff->cb = G_CALLBACK (callback);
  stuff->userdata = userdata;
  return dbus_g_proxy_begin_call (proxy, "QueryTrash", xfdesktop_trash_proxy_query_trash_async_callback, stuff, _dbus_glib_async_data_free, G_TYPE_INVALID);
}
#endif /* defined DBUS_GLIB_CLIENT_WRAPPERS_xfdesktop_trash_proxy */

G_END_DECLS
