/* automatically generated from xfdesktop-settings-ui.glade */
#ifdef __SUNPRO_C
#pragma align 4 (xfdesktop_settings_ui)
#endif
#ifdef __GNUC__
static const char xfdesktop_settings_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfdesktop_settings_ui[] =
#endif
{
  "<?xml version=\"1.0\"?>\n<!--Generated with glade3 3.4.5 on Sun Nov 16 "
  "16:39:43 2008 -->\n<interface>\n  <object class=\"GtkAdjustment\" id=\""
  "adjustment3\">\n    <property name=\"upper\">192</property>\n    <prope"
  "rty name=\"lower\">8</property>\n    <property name=\"page_increment\">"
  "10</property>\n    <property name=\"step_increment\">1</property>\n    "
  "<property name=\"page_size\">0</property>\n    <property name=\"value\""
  ">48</property>\n  </object>\n  <object class=\"GtkAdjustment\" id=\"adj"
  "ustment4\">\n    <property name=\"upper\">144</property>\n    <property"
  " name=\"lower\">0</property>\n    <property name=\"page_increment\">10<"
  "/property>\n    <property name=\"step_increment\">1</property>\n    <pr"
  "operty name=\"page_size\">0</property>\n    <property name=\"value\">12"
  "</property>\n  </object>\n  <object class=\"GtkAdjustment\" id=\"adjust"
  "ment_tooltip\">\n    <property name=\"upper\">256</property>\n    <prop"
  "erty name=\"lower\">0</property>\n    <property name=\"page_increment\""
  ">10</property>\n    <property name=\"step_increment\">1</property>\n   "
  " <property name=\"page_size\">0</property>\n    <property name=\"value\""
  ">128</property>\n  </object>\n  <object class=\"GtkListStore\" id=\"mod"
  "el3\">\n    <columns>\n      <column type=\"gchararray\"/>\n    </colum"
  "ns>\n    <data>\n      <row>\n        <col id=\"0\" translatable=\"yes\""
  ">Left</col>\n      </row>\n      <row>\n        <col id=\"0\" translata"
  "ble=\"yes\">Middle</col>\n      </row>\n      <row>\n        <col id=\""
  "0\" translatable=\"yes\">Right</col>\n      </row>\n    </data>\n  </ob"
  "ject>\n  <object class=\"GtkListStore\" id=\"model4\">\n    <columns>\n"
  "      <column type=\"gchararray\"/>\n    </columns>\n    <data>\n      "
  "<row>\n        <col id=\"0\" translatable=\"yes\">None</col>\n      </r"
  "ow>\n      <row>\n        <col id=\"0\" translatable=\"yes\">Shift</col"
  ">\n      </row>\n      <row>\n        <col id=\"0\" translatable=\"yes\""
  ">Alt</col>\n      </row>\n      <row>\n        <col id=\"0\" translatab"
  "le=\"yes\">Control</col>\n      </row>\n    </data>\n  </object>\n  <ob"
  "ject class=\"GtkListStore\" id=\"model5\">\n    <columns>\n      <colum"
  "n type=\"gchararray\"/>\n    </columns>\n    <data>\n      <row>\n     "
  "   <col id=\"0\" translatable=\"yes\">Left</col>\n      </row>\n      <"
  "row>\n        <col id=\"0\" translatable=\"yes\">Middle</col>\n      </"
  "row>\n      <row>\n        <col id=\"0\" translatable=\"yes\">Right</co"
  "l>\n      </row>\n    </data>\n  </object>\n  <object class=\"GtkListSt"
  "ore\" id=\"model6\">\n    <columns>\n      <column type=\"gchararray\"/"
  ">\n    </columns>\n    <data>\n      <row>\n        <col id=\"0\" trans"
  "latable=\"yes\">None</col>\n      </row>\n      <row>\n        <col id="
  "\"0\" translatable=\"yes\">Shift</col>\n      </row>\n      <row>\n    "
  "    <col id=\"0\" translatable=\"yes\">Alt</col>\n      </row>\n      <"
  "row>\n        <col id=\"0\" translatable=\"yes\">Control</col>\n      <"
  "/row>\n    </data>\n  </object>\n  <object class=\"GtkListStore\" id=\""
  "model7\">\n    <columns>\n      <column type=\"gchararray\"/>\n    </co"
  "lumns>\n    <data>\n      <row>\n        <col id=\"0\" translatable=\"y"
  "es\">None</col>\n      </row>\n      <row>\n        <col id=\"0\" trans"
  "latable=\"yes\">Minimized application icons</col>\n      </row>\n      "
  "<row>\n        <col id=\"0\" translatable=\"yes\">File/launcher icons</"
  "col>\n      </row>\n    </data>\n  </object>\n  <object class=\"XfceTit"
  "ledDialog\" id=\"prefs_dialog\">\n    <property name=\"events\">GDK_POI"
  "NTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK"
  " | GDK_BUTTON_RELEASE_MASK</property>\n    <property name=\"title\" tra"
  "nslatable=\"yes\">Desktop</property>\n    <property name=\"window_posit"
  "ion\">GTK_WIN_POS_CENTER_ON_PARENT</property>\n    <property name=\"ico"
  "n_name\">preferences-desktop-wallpaper</property>\n    <property name=\""
  "type_hint\">GDK_WINDOW_TYPE_HINT_DIALOG</property>\n    <property name="
  "\"subtitle\" translatable=\"yes\">Set desktop background and menu and i"
  "con behavior</property>\n    <property name=\"default-width\">640</prop"
  "erty>\n    <property name=\"default-height\">480</property>\n    <child"
  " internal-child=\"vbox\">\n      <object class=\"GtkVBox\" id=\"dialog-"
  "vbox1\">\n        <property name=\"visible\">True</property>\n        <"
  "property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_H"
  "INT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n"
  "        <property name=\"spacing\">2</property>\n        <child>\n     "
  "     <object class=\"GtkAlignment\" id=\"alignment1\">\n            <pr"
  "operty name=\"visible\">True</property>\n            <child>\n         "
  "     <object class=\"GtkNotebook\" id=\"notebook_settings\">\n         "
  "       <property name=\"visible\">True</property>\n                <pro"
  "perty name=\"can_focus\">True</property>\n                <property nam"
  "e=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | G"
  "DK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n            "
  "    <property name=\"border_width\">6</property>\n                <chil"
  "d>\n                  <object class=\"GtkNotebook\" id=\"notebook_scree"
  "ns\">\n                    <property name=\"visible\">True</property>\n"
  "                    <property name=\"can_focus\">True</property>\n     "
  "               <property name=\"show-border\">False</property>\n       "
  "           </object>\n                </child>\n                <child "
  "type=\"tab\">\n                  <object class=\"GtkLabel\" id=\"label1"
  "\">\n                    <property name=\"visible\">True</property>\n  "
  "                  <property name=\"events\">GDK_POINTER_MOTION_MASK | G"
  "DK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEAS"
  "E_MASK</property>\n                    <property name=\"label\" transla"
  "table=\"yes\">_Background</property>\n                    <property nam"
  "e=\"use_underline\">True</property>\n                  </object>\n     "
  "             <packing>\n                    <property name=\"tab_fill\""
  ">False</property>\n                  </packing>\n                </chil"
  "d>\n                <child>\n                  <object class=\"GtkVBox\""
  " id=\"vbox4\">\n                    <property name=\"visible\">True</pr"
  "operty>\n                    <property name=\"events\">GDK_POINTER_MOTI"
  "ON_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BU"
  "TTON_RELEASE_MASK</property>\n                    <property name=\"bord"
  "er_width\">12</property>\n                    <property name=\"spacing\""
  ">6</property>\n                    <child>\n                      <obje"
  "ct class=\"GtkFrame\" id=\"frame4\">\n                        <property"
  " name=\"visible\">True</property>\n                        <property na"
  "me=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | "
  "GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n           "
  "             <property name=\"label_xalign\">0</property>\n            "
  "            <property name=\"shadow_type\">GTK_SHADOW_NONE</property>\n"
  "                        <child>\n                          <object clas"
  "s=\"GtkAlignment\" id=\"alignment5\">\n                            <pro"
  "perty name=\"visible\">True</property>\n                            <pr"
  "operty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HIN"
  "T_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n  "
  "                          <property name=\"border_width\">6</property>\n"
  "                            <property name=\"left_padding\">6</property"
  ">\n                            <child>\n                              <"
  "object class=\"GtkVBox\" id=\"vbox7\">\n                               "
  " <property name=\"visible\">True</property>\n                          "
  "      <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_M"
  "OTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</prop"
  "erty>\n                                <property name=\"spacing\">6</pr"
  "operty>\n                                <child>\n                     "
  "             <object class=\"GtkCheckButton\" id=\"chk_show_desktop_men"
  "u\">\n                                    <property name=\"visible\">Tr"
  "ue</property>\n                                    <property name=\"can"
  "_focus\">True</property>\n                                    <property"
  " name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK"
  " | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n        "
  "                            <property name=\"label\" translatable=\"yes"
  "\">Include applications menu on _desktop right click</property>\n      "
  "                              <property name=\"use_underline\">True</pr"
  "operty>\n                                    <property name=\"active\">"
  "True</property>\n                                    <property name=\"d"
  "raw_indicator\">True</property>\n                                  </ob"
  "ject>\n                                </child>\n                      "
  "          <child>\n                                  <object class=\"Gt"
  "kAlignment\" id=\"box_menu_subopts\">\n                                "
  "    <property name=\"visible\">True</property>\n                       "
  "             <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_PO"
  "INTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MAS"
  "K</property>\n                                    <property name=\"bott"
  "om_padding\">6</property>\n                                    <propert"
  "y name=\"left_padding\">12</property>\n                                "
  "    <child>\n                                      <object class=\"GtkV"
  "Box\" id=\"vbox10\">\n                                        <property"
  " name=\"visible\">True</property>\n                                    "
  "    <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOT"
  "ION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</proper"
  "ty>\n                                        <property name=\"spacing\""
  ">6</property>\n                                        <child>\n       "
  "                                   <object class=\"GtkHBox\" id=\"hbox1"
  "1\">\n                                            <property name=\"even"
  "ts\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTO"
  "N_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                    "
  "                        <property name=\"spacing\">12</property>\n     "
  "                                       <child>\n                       "
  "                       <object class=\"GtkLabel\" id=\"label17\">\n    "
  "                                            <property name=\"visible\">"
  "True</property>\n                                                <prope"
  "rty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_M"
  "ASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n     "
  "                                           <property name=\"label\" tra"
  "nslatable=\"yes\">_Button:</property>\n                                "
  "                <property name=\"use_underline\">True</property>\n     "
  "                                           <property name=\"mnemonic_wi"
  "dget\">combobox1</property>\n                                          "
  "    </object>\n                                              <packing>\n"
  "                                                <property name=\"expand"
  "\">False</property>\n                                              </pa"
  "cking>\n                                            </child>\n         "
  "                                   <child>\n                           "
  "                   <object class=\"GtkComboBox\" id=\"combobox1\">\n   "
  "                                             <property name=\"visible\""
  ">True</property>\n                                                <prop"
  "erty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_"
  "MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n    "
  "                                            <property name=\"model\">mo"
  "del3</property>\n                                                <child"
  ">\n                                                  <object class=\"Gt"
  "kCellRendererText\" id=\"renderer3\"/>\n                               "
  "                   <attributes>\n                                      "
  "              <attribute name=\"text\">0</attribute>\n                 "
  "                                 </attributes>\n                       "
  "                         </child>\n                                    "
  "          </object>\n                                              <pac"
  "king>\n                                                <property name=\""
  "expand\">False</property>\n                                            "
  "    <property name=\"position\">1</property>\n                         "
  "                     </packing>\n                                      "
  "      </child>\n                                            <child>\n  "
  "                                            <object class=\"GtkLabel\" "
  "id=\"label18\">\n                                                <prope"
  "rty name=\"visible\">True</property>\n                                 "
  "               <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_"
  "POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_M"
  "ASK</property>\n                                                <proper"
  "ty name=\"label\" translatable=\"yes\">Mo_difier:</property>\n         "
  "                                       <property name=\"use_underline\""
  ">True</property>\n                                                <prop"
  "erty name=\"mnemonic_widget\">combobox2</property>\n                   "
  "                           </object>\n                                 "
  "             <packing>\n                                               "
  " <property name=\"expand\">False</property>\n                          "
  "                      <property name=\"position\">2</property>\n       "
  "                                       </packing>\n                    "
  "                        </child>\n                                     "
  "       <child>\n                                              <object c"
  "lass=\"GtkComboBox\" id=\"combobox2\">\n                               "
  "                 <property name=\"visible\">True</property>\n          "
  "                                      <property name=\"events\">GDK_POI"
  "NTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK"
  " | GDK_BUTTON_RELEASE_MASK</property>\n                                "
  "                <property name=\"model\">model4</property>\n           "
  "                                     <child>\n                         "
  "                         <object class=\"GtkCellRendererText\" id=\"ren"
  "derer4\"/>\n                                                  <attribut"
  "es>\n                                                    <attribute nam"
  "e=\"text\">0</attribute>\n                                             "
  "     </attributes>\n                                                </c"
  "hild>\n                                              </object>\n       "
  "                                       <packing>\n                     "
  "                           <property name=\"expand\">False</property>\n"
  "                                                <property name=\"positi"
  "on\">3</property>\n                                              </pack"
  "ing>\n                                            </child>\n           "
  "                               </object>\n                             "
  "           </child>\n                                        <child>\n "
  "                                         <object class=\"GtkCheckButton"
  "\" id=\"chk_menu_show_app_icons\">\n                                   "
  "         <property name=\"visible\">True</property>\n                  "
  "                          <property name=\"can_focus\">True</property>\n"
  "                                            <property name=\"events\">G"
  "DK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRES"
  "S_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                          "
  "                  <property name=\"label\" translatable=\"yes\">Show _a"
  "pplication icons in menu</property>\n                                  "
  "          <property name=\"use_underline\">True</property>\n           "
  "                                 <property name=\"active\">True</proper"
  "ty>\n                                            <property name=\"draw_"
  "indicator\">True</property>\n                                          "
  "</object>\n                                          <packing>\n       "
  "                                     <property name=\"position\">1</pro"
  "perty>\n                                          </packing>\n         "
  "                               </child>\n                              "
  "          <child>\n                                          <object cl"
  "ass=\"GtkHBox\" id=\"hbox12\">\n                                       "
  "     <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MO"
  "TION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</prope"
  "rty>\n                                            <property name=\"spac"
  "ing\">12</property>\n                                            <child"
  ">\n                                              <object class=\"GtkBut"
  "ton\" id=\"button5\">\n                                                "
  "<property name=\"visible\">True</property>\n                           "
  "                     <property name=\"can_focus\">True</property>\n    "
  "                                            <property name=\"receives_d"
  "efault\">True</property>\n                                             "
  "   <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTI"
  "ON_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</propert"
  "y>\n                                                <property name=\"la"
  "bel\" translatable=\"yes\">_Edit desktop menu</property>\n             "
  "                                   <property name=\"use_underline\">Tru"
  "e</property>\n                                              </object>\n"
  "                                              <packing>\n              "
  "                                  <property name=\"expand\">False</prop"
  "erty>\n                                              </packing>\n      "
  "                                      </child>\n                       "
  "                   </object>\n                                         "
  " <packing>\n                                            <property name="
  "\"position\">2</property>\n                                          </"
  "packing>\n                                        </child>\n           "
  "                           </object>\n                                 "
  "   </child>\n                                  </object>\n             "
  "                     <packing>\n                                    <pr"
  "operty name=\"position\">1</property>\n                                "
  "  </packing>\n                                </child>\n               "
  "               </object>\n                            </child>\n       "
  "                   </object>\n                        </child>\n       "
  "                 <child type=\"label\">\n                          <obj"
  "ect class=\"GtkLabel\" id=\"label10\">\n                            <pr"
  "operty name=\"visible\">True</property>\n                            <p"
  "roperty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HI"
  "NT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n "
  "                           <property name=\"label\" translatable=\"yes\""
  ">&lt;b&gt;Desktop Menu&lt;/b&gt;</property>\n                          "
  "  <property name=\"use_markup\">True</property>\n                      "
  "    </object>\n                        </child>\n                      "
  "</object>\n                      <packing>\n                        <pr"
  "operty name=\"expand\">False</property>\n                      </packin"
  "g>\n                    </child>\n                    <child>\n        "
  "              <object class=\"GtkFrame\" id=\"frame5\">\n              "
  "          <property name=\"visible\">True</property>\n                 "
  "       <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_"
  "MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</pro"
  "perty>\n                        <property name=\"label_xalign\">0</prop"
  "erty>\n                        <property name=\"shadow_type\">GTK_SHADO"
  "W_NONE</property>\n                        <child>\n                   "
  "       <object class=\"GtkAlignment\" id=\"alignment6\">\n             "
  "               <property name=\"visible\">True</property>\n            "
  "                <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK"
  "_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_"
  "MASK</property>\n                            <property name=\"border_wi"
  "dth\">6</property>\n                            <property name=\"left_p"
  "adding\">6</property>\n                            <child>\n           "
  "                   <object class=\"GtkVBox\" id=\"vbox8\">\n           "
  "                     <property name=\"visible\">True</property>\n      "
  "                          <property name=\"events\">GDK_POINTER_MOTION_"
  "MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTO"
  "N_RELEASE_MASK</property>\n                                <property na"
  "me=\"spacing\">6</property>\n                                <child>\n "
  "                                 <object class=\"GtkCheckButton\" id=\""
  "chk_show_winlist_menu\">\n                                    <property"
  " name=\"visible\">True</property>\n                                    "
  "<property name=\"can_focus\">True</property>\n                         "
  "           <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POIN"
  "TER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK<"
  "/property>\n                                    <property name=\"label\""
  " translatable=\"yes\">Show _window list menu on desktop middle click</p"
  "roperty>\n                                    <property name=\"use_unde"
  "rline\">True</property>\n                                    <property "
  "name=\"active\">True</property>\n                                    <p"
  "roperty name=\"draw_indicator\">True</property>\n                      "
  "            </object>\n                                </child>\n      "
  "                          <child>\n                                  <o"
  "bject class=\"GtkAlignment\" id=\"box_winlist_subopts\">\n             "
  "                       <property name=\"visible\">True</property>\n    "
  "                                <property name=\"events\">GDK_POINTER_M"
  "OTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK"
  "_BUTTON_RELEASE_MASK</property>\n                                    <p"
  "roperty name=\"bottom_padding\">6</property>\n                         "
  "           <property name=\"left_padding\">12</property>\n             "
  "                       <child>\n                                      <"
  "object class=\"GtkVBox\" id=\"vbox11\">\n                              "
  "          <property name=\"visible\">True</property>\n                 "
  "                       <property name=\"events\">GDK_POINTER_MOTION_MAS"
  "K | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_R"
  "ELEASE_MASK</property>\n                                        <proper"
  "ty name=\"spacing\">6</property>\n                                     "
  "   <child>\n                                          <object class=\"G"
  "tkHBox\" id=\"hbox14\">\n                                            <p"
  "roperty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HI"
  "NT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n "
  "                                           <property name=\"spacing\">1"
  "2</property>\n                                            <child>\n    "
  "                                          <object class=\"GtkLabel\" id"
  "=\"label23\">\n                                                <propert"
  "y name=\"visible\">True</property>\n                                   "
  "             <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_PO"
  "INTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MAS"
  "K</property>\n                                                <property"
  " name=\"label\" translatable=\"yes\">B_utton:</property>\n             "
  "                                   <property name=\"use_underline\">Tru"
  "e</property>\n                                                <property"
  " name=\"mnemonic_widget\">combobox8</property>\n                       "
  "                       </object>\n                                     "
  "         <packing>\n                                                <pr"
  "operty name=\"expand\">False</property>\n                              "
  "                </packing>\n                                           "
  " </child>\n                                            <child>\n       "
  "                                       <object class=\"GtkComboBox\" id"
  "=\"combobox8\">\n                                                <prope"
  "rty name=\"visible\">True</property>\n                                 "
  "               <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_"
  "POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_M"
  "ASK</property>\n                                                <proper"
  "ty name=\"model\">model5</property>\n                                  "
  "              <child>\n                                                "
  "  <object class=\"GtkCellRendererText\" id=\"renderer5\"/>\n           "
  "                                       <attributes>\n                  "
  "                                  <attribute name=\"text\">0</attribute"
  ">\n                                                  </attributes>\n   "
  "                                             </child>\n                "
  "                              </object>\n                              "
  "                <packing>\n                                            "
  "    <property name=\"expand\">False</property>\n                       "
  "                         <property name=\"position\">1</property>\n    "
  "                                          </packing>\n                 "
  "                           </child>\n                                  "
  "          <child>\n                                              <objec"
  "t class=\"GtkLabel\" id=\"label24\">\n                                 "
  "               <property name=\"visible\">True</property>\n            "
  "                                    <property name=\"events\">GDK_POINT"
  "ER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK |"
  " GDK_BUTTON_RELEASE_MASK</property>\n                                  "
  "              <property name=\"label\" translatable=\"yes\">Modi_fier:<"
  "/property>\n                                                <property n"
  "ame=\"use_underline\">True</property>\n                                "
  "                <property name=\"mnemonic_widget\">combobox9</property>"
  "\n                                              </object>\n            "
  "                                  <packing>\n                          "
  "                      <property name=\"expand\">False</property>\n     "
  "                                           <property name=\"position\">"
  "2</property>\n                                              </packing>\n"
  "                                            </child>\n                 "
  "                           <child>\n                                   "
  "           <object class=\"GtkComboBox\" id=\"combobox9\">\n           "
  "                                     <property name=\"visible\">True</p"
  "roperty>\n                                                <property nam"
  "e=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | G"
  "DK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n            "
  "                                    <property name=\"model\">model6</pr"
  "operty>\n                                                <child>\n     "
  "                                             <object class=\"GtkCellRen"
  "dererText\" id=\"renderer6\"/>\n                                       "
  "           <attributes>\n                                              "
  "      <attribute name=\"text\">0</attribute>\n                         "
  "                         </attributes>\n                               "
  "                 </child>\n                                            "
  "  </object>\n                                              <packing>\n "
  "                                               <property name=\"expand\""
  ">False</property>\n                                                <pro"
  "perty name=\"position\">3</property>\n                                 "
  "             </packing>\n                                            </"
  "child>\n                                          </object>\n          "
  "                              </child>\n                               "
  "         <child>\n                                          <object cla"
  "ss=\"GtkCheckButton\" id=\"chk_winlist_show_app_icons\">\n             "
  "                               <property name=\"visible\">True</propert"
  "y>\n                                            <property name=\"can_fo"
  "cus\">True</property>\n                                            <pro"
  "perty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT"
  "_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n   "
  "                                         <property name=\"label\" trans"
  "latable=\"yes\">Sh_ow application icons in menu</property>\n           "
  "                                 <property name=\"use_underline\">True<"
  "/property>\n                                            <property name="
  "\"active\">True</property>\n                                           "
  " <property name=\"draw_indicator\">True</property>\n                   "
  "                       </object>\n                                     "
  "     <packing>\n                                            <property n"
  "ame=\"position\">1</property>\n                                        "
  "  </packing>\n                                        </child>\n       "
  "                                 <child>\n                             "
  "             <object class=\"GtkCheckButton\" id=\"chk_show_winlist_ws_"
  "names\">\n                                            <property name=\""
  "visible\">True</property>\n                                            "
  "<property name=\"can_focus\">True</property>\n                         "
  "                   <property name=\"events\">GDK_POINTER_MOTION_MASK | "
  "GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEA"
  "SE_MASK</property>\n                                            <proper"
  "ty name=\"label\" translatable=\"yes\">Show workspace _names in list</p"
  "roperty>\n                                            <property name=\""
  "use_underline\">True</property>\n                                      "
  "      <property name=\"active\">True</property>\n                      "
  "                      <property name=\"draw_indicator\">True</property>"
  "\n                                          </object>\n                "
  "                          <packing>\n                                  "
  "          <property name=\"position\">2</property>\n                   "
  "                       </packing>\n                                    "
  "    </child>\n                                        <child>\n        "
  "                                  <object class=\"GtkAlignment\" id=\"b"
  "ox_winlist_names_subopts\">\n                                          "
  "  <property name=\"visible\">True</property>\n                         "
  "                   <property name=\"events\">GDK_POINTER_MOTION_MASK | "
  "GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEA"
  "SE_MASK</property>\n                                            <proper"
  "ty name=\"left_padding\">12</property>\n                               "
  "             <child>\n                                              <ob"
  "ject class=\"GtkCheckButton\" id=\"chk_show_winlist_ws_submenus\">\n   "
  "                                             <property name=\"visible\""
  ">True</property>\n                                                <prop"
  "erty name=\"can_focus\">True</property>\n                              "
  "                  <property name=\"events\">GDK_POINTER_MOTION_MASK | G"
  "DK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEAS"
  "E_MASK</property>\n                                                <pro"
  "perty name=\"label\" translatable=\"yes\">Use _submenus for the windows"
  " in each workspace</property>\n                                        "
  "        <property name=\"use_underline\">True</property>\n             "
  "                                   <property name=\"draw_indicator\">Tr"
  "ue</property>\n                                              </object>\n"
  "                                            </child>\n                 "
  "                         </object>\n                                   "
  "       <packing>\n                                            <property"
  " name=\"position\">3</property>\n                                      "
  "    </packing>\n                                        </child>\n     "
  "                                   <child>\n                           "
  "               <object class=\"GtkCheckButton\" id=\"chk_show_winlist_s"
  "ticky_once\">\n                                            <property na"
  "me=\"visible\">True</property>\n                                       "
  "     <property name=\"can_focus\">True</property>\n                    "
  "                        <property name=\"events\">GDK_POINTER_MOTION_MA"
  "SK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_"
  "RELEASE_MASK</property>\n                                            <p"
  "roperty name=\"label\" translatable=\"yes\">Show s_ticky windows only i"
  "n active workspace</property>\n                                        "
  "    <property name=\"use_underline\">True</property>\n                 "
  "                           <property name=\"draw_indicator\">True</prop"
  "erty>\n                                          </object>\n           "
  "                               <packing>\n                             "
  "               <property name=\"position\">4</property>\n              "
  "                            </packing>\n                               "
  "         </child>\n                                        <child>\n   "
  "                                       <object class=\"GtkCheckButton\""
  " id=\"chk_show_app_remove_workspaces\">\n                              "
  "              <property name=\"visible\">True</property>\n             "
  "                               <property name=\"can_focus\">True</prope"
  "rty>\n                                            <property name=\"even"
  "ts\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTO"
  "N_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                    "
  "                        <property name=\"label\" translatable=\"yes\">S"
  "how a_dd and remove workspace options in list</property>\n             "
  "                               <property name=\"active\">True</property"
  ">\n                                            <property name=\"use_und"
  "erline\">True</property>\n                                            <"
  "property name=\"draw_indicator\">True</property>\n                     "
  "                     </object>\n                                       "
  "   <packing>\n                                            <property nam"
  "e=\"position\">5</property>\n                                          "
  "</packing>\n                                        </child>\n         "
  "                             </object>\n                               "
  "     </child>\n                                  </object>\n           "
  "                       <packing>\n                                    <"
  "property name=\"position\">1</property>\n                              "
  "    </packing>\n                                </child>\n             "
  "                 </object>\n                            </child>\n     "
  "                     </object>\n                        </child>\n     "
  "                   <child type=\"label\">\n                          <o"
  "bject class=\"GtkLabel\" id=\"label11\">\n                            <"
  "property name=\"visible\">True</property>\n                            "
  "<property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_"
  "HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n"
  "                            <property name=\"label\" translatable=\"yes"
  "\">&lt;b&gt;Window List Menu&lt;/b&gt;</property>\n                    "
  "        <property name=\"use_markup\">True</property>\n                "
  "          </object>\n                        </child>\n                "
  "      </object>\n                      <packing>\n                     "
  "   <property name=\"expand\">False</property>\n                        "
  "<property name=\"position\">1</property>\n                      </packi"
  "ng>\n                    </child>\n                  </object>\n       "
  "         </child>\n                <child type=\"tab\">\n              "
  "    <object class=\"GtkLabel\" id=\"label2\">\n                    <pro"
  "perty name=\"visible\">True</property>\n                    <property n"
  "ame=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |"
  " GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n          "
  "          <property name=\"label\" translatable=\"yes\">_Menus</propert"
  "y>\n                    <property name=\"use_underline\">True</property"
  ">\n                  </object>\n                  <packing>\n          "
  "          <property name=\"position\">1</property>\n                   "
  " <property name=\"tab_fill\">False</property>\n                  </pack"
  "ing>\n                </child>\n                <child>\n              "
  "    <object class=\"GtkVBox\" id=\"vbox9\">\n                    <prope"
  "rty name=\"visible\">True</property>\n                    <property nam"
  "e=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | G"
  "DK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n            "
  "        <property name=\"border_width\">12</property>\n                "
  "    <property name=\"spacing\">6</property>\n                    <child"
  ">\n                      <object class=\"GtkFrame\" id=\"frame6\">\n   "
  "                     <property name=\"visible\">True</property>\n      "
  "                  <property name=\"events\">GDK_POINTER_MOTION_MASK | G"
  "DK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEAS"
  "E_MASK</property>\n                        <property name=\"label_xalig"
  "n\">0</property>\n                        <property name=\"shadow_type\""
  ">GTK_SHADOW_NONE</property>\n                        <child>\n         "
  "                 <object class=\"GtkAlignment\" id=\"alignment11\">\n  "
  "                          <property name=\"visible\">True</property>\n "
  "                           <property name=\"events\">GDK_POINTER_MOTION"
  "_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTT"
  "ON_RELEASE_MASK</property>\n                            <property name="
  "\"border_width\">6</property>\n                            <property na"
  "me=\"left_padding\">6</property>\n                            <child>\n"
  "                              <object class=\"GtkVBox\" id=\"vbox6\">\n"
  "                                <property name=\"visible\">True</proper"
  "ty>\n                                <property name=\"events\">GDK_POIN"
  "TER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK "
  "| GDK_BUTTON_RELEASE_MASK</property>\n                                <"
  "property name=\"spacing\">6</property>\n                               "
  " <child>\n                                  <object class=\"GtkHBox\" i"
  "d=\"hbox7\">\n                                    <property name=\"visi"
  "ble\">True</property>\n                                    <property na"
  "me=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | "
  "GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n           "
  "                         <property name=\"spacing\">12</property>\n    "
  "                                <child>\n                              "
  "        <object class=\"GtkLabel\" id=\"label12\">\n                   "
  "                     <property name=\"visible\">True</property>\n      "
  "                                  <property name=\"events\">GDK_POINTER"
  "_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | G"
  "DK_BUTTON_RELEASE_MASK</property>\n                                    "
  "    <property name=\"label\" translatable=\"yes\">Icon _type:</property"
  ">\n                                        <property name=\"use_underli"
  "ne\">True</property>\n                                      </object>\n"
  "                                      <packing>\n                      "
  "                  <property name=\"expand\">False</property>\n         "
  "                             </packing>\n                              "
  "      </child>\n                                    <child>\n          "
  "                            <object class=\"GtkComboBox\" id=\"combo_ic"
  "ons\">\n                                        <property name=\"visibl"
  "e\">True</property>\n                                        <property "
  "name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK "
  "| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n         "
  "                               <property name=\"model\">model7</propert"
  "y>\n                                        <child>\n                  "
  "                        <object class=\"GtkCellRendererText\" id=\"rend"
  "erer7\"/>\n                                          <attributes>\n    "
  "                                        <attribute name=\"text\">0</att"
  "ribute>\n                                          </attributes>\n     "
  "                                   </child>\n                          "
  "            </object>\n                                      <packing>\n"
  "                                        <property name=\"expand\">False"
  "</property>\n                                        <property name=\"p"
  "osition\">1</property>\n                                      </packing"
  ">\n                                    </child>\n                      "
  "            </object>\n                                  <packing>\n   "
  "                                 <property name=\"expand\">False</prope"
  "rty>\n                                  </packing>\n                   "
  "             </child>\n                                <child>\n       "
  "                           <object class=\"GtkHBox\" id=\"hbox8\">\n   "
  "                                 <property name=\"visible\">True</prope"
  "rty>\n                                    <property name=\"events\">GDK"
  "_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_"
  "MASK | GDK_BUTTON_RELEASE_MASK</property>\n                            "
  "        <property name=\"spacing\">12</property>\n                     "
  "               <child>\n                                      <object c"
  "lass=\"GtkLabel\" id=\"label13\">\n                                    "
  "    <property name=\"visible\">True</property>\n                       "
  "                 <property name=\"events\">GDK_POINTER_MOTION_MASK | GD"
  "K_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE"
  "_MASK</property>\n                                        <property nam"
  "e=\"label\" translatable=\"yes\">Icon _size:</property>\n              "
  "                          <property name=\"use_underline\">True</proper"
  "ty>\n                                      </object>\n                 "
  "                     <packing>\n                                       "
  " <property name=\"expand\">False</property>\n                          "
  "            </packing>\n                                    </child>\n "
  "                                   <child>\n                           "
  "           <object class=\"GtkSpinButton\" id=\"spin_icon_size\">\n    "
  "                                    <property name=\"visible\">True</pr"
  "operty>\n                                        <property name=\"can_f"
  "ocus\">True</property>\n                                        <proper"
  "ty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MA"
  "SK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n      "
  "                                  <property name=\"adjustment\">adjustm"
  "ent3</property>\n                                      </object>\n     "
  "                                 <packing>\n                           "
  "             <property name=\"expand\">False</property>\n              "
  "                          <property name=\"position\">1</property>\n   "
  "                                   </packing>\n                        "
  "            </child>\n                                  </object>\n    "
  "                              <packing>\n                              "
  "      <property name=\"expand\">False</property>\n                     "
  "               <property name=\"position\">1</property>\n              "
  "                    </packing>\n                                </child"
  ">\n                                <child>\n                           "
  "       <object class=\"GtkCheckButton\" id=\"chk_show_thumbnails\">\n  "
  "                                  <property name=\"visible\">True</prop"
  "erty>\n                                    <property name=\"can_focus\""
  ">True</property>\n                                    <property name=\""
  "events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_B"
  "UTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                "
  "                    <property name=\"label\" translatable=\"yes\">Show "
  "t_humbnails</property>\n                                    <property n"
  "ame=\"tooltip-text\" translatable=\"yes\">Select this option to display"
  " preview-able files on the desktop as automatically generated thumbnail"
  " icons.</property>\n                                    <property name="
  "\"use_underline\">True</property>\n                                    "
  "<property name=\"draw_indicator\">True</property>\n                    "
  "              </object>\n                                </child>\n    "
  "                            <child>\n                                  "
  "<object class=\"GtkCheckButton\" id=\"chk_show_hidden_files\">\n       "
  "                             <property name=\"visible\">True</property>"
  "\n                                    <property name=\"can_focus\">True"
  "</property>\n                                    <property name=\"event"
  "s\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON"
  "_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                     "
  "               <property name=\"label\" translatable=\"yes\">Show hidde"
  "n files on the desktop</property>\n                                    "
  "<property name=\"use_underline\">True</property>\n                     "
  "               <property name=\"draw_indicator\">True</property>\n     "
  "                             </object>\n                               "
  " </child>\n                                <child>\n                   "
  "               <object class=\"GtkCheckButton\" id=\"chk_single_click\""
  ">\n                                    <property name=\"visible\">True<"
  "/property>\n                                    <property name=\"can_fo"
  "cus\">True</property>\n                                    <property na"
  "me=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | "
  "GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n           "
  "                         <property name=\"label\" translatable=\"yes\">"
  "Single _click to activate items</property>\n                           "
  "         <property name=\"use_underline\">True</property>\n            "
  "                        <property name=\"draw_indicator\">True</propert"
  "y>\n                                  </object>\n                      "
  "          </child>\n                                <child>\n          "
  "                        <object class=\"GtkHBox\" id=\"hboxtooltip\">\n"
  "                                    <property name=\"visible\">True</pr"
  "operty>\n                                    <property name=\"events\">"
  "GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRE"
  "SS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                         "
  "           <property name=\"spacing\">4</property>\n                   "
  "                 <child>\n                                      <object"
  " class=\"GtkCheckButton\" id=\"chk_show_tooltips\">\n                  "
  "                      <property name=\"visible\">True</property>\n     "
  "                                   <property name=\"can_focus\">True</p"
  "roperty>\n                                        <property name=\"even"
  "ts\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTO"
  "N_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                    "
  "                    <property name=\"label\" translatable=\"yes\">Show "
  "icon tooltips. Size:</property>\n                                      "
  "  <property name=\"use_underline\">True</property>\n                   "
  "                     <property name=\"draw_indicator\">True</property>\n"
  "                                        <property name=\"active\">True<"
  "/property>\n                                      </object>\n          "
  "                            <packing>\n                                "
  "        <property name=\"expand\">False</property>\n                   "
  "                   </packing>\n                                    </ch"
  "ild>\n                                    <child>\n                    "
  "                  <object class=\"GtkSpinButton\" id=\"spin_tooltip_siz"
  "e\">\n                                        <property name=\"visible\""
  ">True</property>\n                                        <property nam"
  "e=\"sensitive\">True</property>\n                                      "
  "  <property name=\"can_focus\">True</property>\n                       "
  "                 <property name=\"events\">GDK_POINTER_MOTION_MASK | GD"
  "K_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE"
  "_MASK</property>\n                                        <property nam"
  "e=\"adjustment\">adjustment_tooltip</property>\n                       "
  "                 <property name=\"tooltip-text\" translatable=\"yes\">S"
  "ize of the tooltip preview image.</property>\n                         "
  "             </object>\n                                      <packing>"
  "\n                                        <property name=\"expand\">Fal"
  "se</property>\n                                        <property name=\""
  "position\">1</property>\n                                      </packin"
  "g>\n                                    </child>\n                     "
  "             </object>\n                                  <packing>\n  "
  "                                  <property name=\"expand\">False</prop"
  "erty>\n                                    <property name=\"position\">"
  "2</property>\n                                  </packing>\n           "
  "                     </child>\n                                <child>\n"
  "                                  <object class=\"GtkHBox\" id=\"hbox9\""
  ">\n                                    <property name=\"visible\">True<"
  "/property>\n                                    <property name=\"events"
  "\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_"
  "PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                      "
  "              <property name=\"spacing\">4</property>\n                "
  "                    <child>\n                                      <obj"
  "ect class=\"GtkCheckButton\" id=\"chk_custom_font_size\">\n            "
  "                            <property name=\"visible\">True</property>\n"
  "                                        <property name=\"can_focus\">Tr"
  "ue</property>\n                                        <property name=\""
  "events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_B"
  "UTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                "
  "                        <property name=\"label\" translatable=\"yes\">U"
  "se custom _font size:</property>\n                                     "
  "   <property name=\"use_underline\">True</property>\n                  "
  "                      <property name=\"draw_indicator\">True</property>"
  "\n                                      </object>\n                    "
  "                  <packing>\n                                        <p"
  "roperty name=\"expand\">False</property>\n                             "
  "         </packing>\n                                    </child>\n    "
  "                                <child>\n                              "
  "        <object class=\"GtkSpinButton\" id=\"spin_font_size\">\n       "
  "                                 <property name=\"visible\">True</prope"
  "rty>\n                                        <property name=\"sensitiv"
  "e\">False</property>\n                                        <property"
  " name=\"can_focus\">True</property>\n                                  "
  "      <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_M"
  "OTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</prop"
  "erty>\n                                        <property name=\"adjustm"
  "ent\">adjustment4</property>\n                                      </o"
  "bject>\n                                      <packing>\n              "
  "                          <property name=\"expand\">False</property>\n "
  "                                       <property name=\"position\">1</p"
  "roperty>\n                                      </packing>\n           "
  "                         </child>\n                                  </"
  "object>\n                                  <packing>\n                 "
  "                   <property name=\"expand\">False</property>\n        "
  "                            <property name=\"position\">2</property>\n "
  "                                 </packing>\n                          "
  "      </child>\n                              </object>\n              "
  "              </child>\n                          </object>\n          "
  "              </child>\n                        <child type=\"label\">\n"
  "                          <object class=\"GtkLabel\" id=\"label14\">\n "
  "                           <property name=\"visible\">True</property>\n"
  "                            <property name=\"events\">GDK_POINTER_MOTIO"
  "N_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUT"
  "TON_RELEASE_MASK</property>\n                            <property name"
  "=\"label\" translatable=\"yes\">&lt;b&gt;Appearance&lt;/b&gt;</property"
  ">\n                            <property name=\"use_markup\">True</prop"
  "erty>\n                          </object>\n                        </c"
  "hild>\n                      </object>\n                      <packing>"
  "\n                        <property name=\"expand\">False</property>\n "
  "                     </packing>\n                    </child>\n        "
  "            <child>\n                      <object class=\"GtkFrame\" i"
  "d=\"frame7\">\n                        <property name=\"visible\">True<"
  "/property>\n                        <property name=\"events\">GDK_POINT"
  "ER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK |"
  " GDK_BUTTON_RELEASE_MASK</property>\n                        <property "
  "name=\"label_xalign\">0</property>\n                        <property n"
  "ame=\"shadow_type\">GTK_SHADOW_NONE</property>\n                       "
  " <child>\n                          <object class=\"GtkAlignment\" id=\""
  "alignment12\">\n                            <property name=\"visible\">"
  "True</property>\n                            <property name=\"events\">"
  "GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRE"
  "SS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                         "
  "   <property name=\"border_width\">6</property>\n                      "
  "      <property name=\"left_padding\">6</property>\n                   "
  "         <child>\n                              <object class=\"GtkScro"
  "lledWindow\" id=\"scrolledwindow2\">\n                                <"
  "property name=\"visible\">True</property>\n                            "
  "    <property name=\"can_focus\">True</property>\n                     "
  "           <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POIN"
  "TER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK<"
  "/property>\n                                <property name=\"hscrollbar"
  "_policy\">GTK_POLICY_AUTOMATIC</property>\n                            "
  "    <property name=\"vscrollbar_policy\">GTK_POLICY_AUTOMATIC</property"
  ">\n                                <property name=\"shadow_type\">GTK_S"
  "HADOW_ETCHED_IN</property>\n                                <child>\n  "
  "                                <object class=\"GtkTreeView\" id=\"tree"
  "view_default_icons\">\n                                    <property na"
  "me=\"visible\">True</property>\n                                    <pr"
  "operty name=\"can_focus\">True</property>\n                            "
  "        <property name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER"
  "_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</pr"
  "operty>\n                                    <property name=\"headers_v"
  "isible\">False</property>\n                                  </object>\n"
  "                                </child>\n                             "
  " </object>\n                            </child>\n                     "
  "     </object>\n                        </child>\n                     "
  "   <child type=\"label\">\n                          <object class=\"Gt"
  "kLabel\" id=\"label15\">\n                            <property name=\""
  "visible\">True</property>\n                            <property name=\""
  "events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_B"
  "UTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                "
  "            <property name=\"label\" translatable=\"yes\">&lt;b&gt;Defa"
  "ult Icons&lt;/b&gt;</property>\n                            <property n"
  "ame=\"use_markup\">True</property>\n                          </object>"
  "\n                        </child>\n                      </object>\n  "
  "                    <packing>\n                        <property name=\""
  "position\">1</property>\n                      </packing>\n            "
  "        </child>\n                  </object>\n                </child>"
  "\n                <child type=\"tab\">\n                  <object class"
  "=\"GtkLabel\" id=\"label3\">\n                    <property name=\"visi"
  "ble\">True</property>\n                    <property name=\"events\">GD"
  "K_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS"
  "_MASK | GDK_BUTTON_RELEASE_MASK</property>\n                    <proper"
  "ty name=\"label\" translatable=\"yes\">_Icons</property>\n             "
  "       <property name=\"use_underline\">True</property>\n              "
  "    </object>\n                  <packing>\n                    <proper"
  "ty name=\"position\">2</property>\n                    <property name=\""
  "tab_fill\">False</property>\n                  </packing>\n            "
  "    </child>\n              </object>\n            </child>\n          "
  "</object>\n          <packing>\n            <property name=\"position\""
  ">1</property>\n          </packing>\n        </child>\n        <child i"
  "nternal-child=\"action_area\">\n          <object class=\"GtkHButtonBox"
  "\" id=\"dialog-action_area1\">\n            <property name=\"visible\">"
  "True</property>\n            <property name=\"events\">GDK_POINTER_MOTI"
  "ON_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BU"
  "TTON_RELEASE_MASK</property>\n            <child>\n              <objec"
  "t class=\"GtkButton\" id=\"button2\">\n                <property name=\""
  "visible\">True</property>\n                <property name=\"can_focus\""
  ">True</property>\n                <property name=\"receives_default\">T"
  "rue</property>\n                <property name=\"events\">GDK_POINTER_M"
  "OTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK"
  "_BUTTON_RELEASE_MASK</property>\n                <property name=\"label"
  "\">gtk-help</property>\n                <property name=\"use_stock\">Tr"
  "ue</property>\n              </object>\n            </child>\n         "
  "   <child>\n              <object class=\"GtkButton\" id=\"bnt_exit\">\n"
  "                <property name=\"visible\">True</property>\n           "
  "     <property name=\"can_focus\">True</property>\n                <pro"
  "perty name=\"receives_default\">True</property>\n                <prope"
  "rty name=\"events\">GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_M"
  "ASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK</property>\n     "
  "           <property name=\"label\">gtk-close</property>\n             "
  "   <property name=\"use_stock\">True</property>\n              </object"
  ">\n              <packing>\n                <property name=\"position\""
  ">1</property>\n              </packing>\n            </child>\n        "
  "  </object>\n          <packing>\n            <property name=\"expand\""
  ">False</property>\n            <property name=\"pack_type\">GTK_PACK_EN"
  "D</property>\n          </packing>\n        </child>\n      </object>\n"
  "    </child>\n    <action-widgets>\n      <action-widget response=\"-11"
  "\">button2</action-widget>\n      <action-widget response=\"-3\">bnt_ex"
  "it</action-widget>\n    </action-widgets>\n  </object>\n</interface>\n"
};

static const unsigned xfdesktop_settings_ui_length = 58887u;

