


#ifndef __XFCE_DESKTOP_ENUM_TYPES_H__
#define __XFCE_DESKTOP_ENUM_TYPES_H__
G_BEGIN_DECLS
/* enumerations from "xfce-backdrop.h" */
GType xfce_backdrop_image_style_get_type (void) G_GNUC_CONST;
#define XFCE_TYPE_BACKDROP_IMAGE_STYLE (xfce_backdrop_image_style_get_type())
GType xfce_backdrop_color_style_get_type (void) G_GNUC_CONST;
#define XFCE_TYPE_BACKDROP_COLOR_STYLE (xfce_backdrop_color_style_get_type())
GType xfce_backdrop_cycle_period_get_type (void) G_GNUC_CONST;
#define XFCE_TYPE_BACKDROP_CYCLE_PERIOD (xfce_backdrop_cycle_period_get_type())
/* enumerations from "xfce-desktop.h" */
GType xfce_desktop_icon_style_get_type (void) G_GNUC_CONST;
#define XFCE_TYPE_DESKTOP_ICON_STYLE (xfce_desktop_icon_style_get_type())
G_END_DECLS

#endif /* !__XFCE_DESKTOP_ENUM_TYPES_H__ */



